@@ -764,6 +764,235 @@ static PyObject *eikonalc_oneshot(PyObject *self, PyObject *args){
 	
 }
 
+static PyObject *eikonalc_oneshot_angle(PyObject *self, PyObject *args){
+
+    /*Below is the input part*/
+    float f1,f2,f3,f4,f5,f6,f7,f8,f9;
+    int f10,f11,f12,f13;
+    
+	/**initialize data input**/
+    PyObject *arg1=NULL;
+    PyObject *arr1=NULL;
+    int nd;
+
+	PyArg_ParseTuple(args, "Offfffffffiiii", &arg1, &f1, &f2, &f3, &f4, &f5, &f6, &f7, &f8, &f9, &f10, &f11, &f12, &f13);
+
+    int b1, b2, b3, n1, n2, n3, nshot, ndim, i, is,order,n123, *p;
+    float br1, br2, br3, o1, o2, o3, d1, d2, d3;
+    float **s, *t, *v, *dip, *azim;
+    float x, y, z;
+    bool plane[3];
+    
+	x=f1;
+	y=f2;
+	z=f3;
+	
+	o1=f4;
+	o2=f5;
+	o3=f6;
+	
+	d1=f7;
+	d2=f8;
+	d3=f9;
+	
+	n1=f10;
+	n2=f11;
+	n3=f12;
+	
+	order=f13;
+	
+	
+    arr1 = PyArray_FROM_OTF(arg1, NPY_FLOAT, NPY_IN_ARRAY);
+    /*
+     * my code starts here
+     */
+    nd=PyArray_NDIM(arr1);
+    npy_intp *sp=PyArray_SHAPE(arr1);
+
+	br1=d1;
+	br2=d2;
+	br3=d3;
+	plane[2]=false;
+	plane[1]=false;
+	plane[0]=false;
+	b1= plane[2]? n1: (int) (br1/d1+0.5); 
+	b2= plane[1]? n2: (int) (br2/d2+0.5);
+	b3= plane[0]? n3: (int) (br3/d3+0.5); 
+
+    if( b1<1 ) b1=1;  
+    if( b2<1 ) b2=1;  
+    if( b3<1 ) b3=1;
+
+    /* File with shot locations (n2=number of shots, n1=3) */
+
+	nshot = 1;
+	ndim = 3;
+     
+    s = (float**)malloc(nshot * sizeof(float*));
+    for (int i = 0; i < nshot; i++)
+        s[i] = (float*)malloc(ndim * sizeof(float));
+
+	s[0][0]=x;
+	s[0][1]=y;
+	s[0][2]=z;
+	
+    n123 = n1*n2*n3;
+
+	t = (float*)malloc(n123 * sizeof(float));
+	dip = (float*)malloc(n123 * sizeof(float));
+	azim = (float*)malloc(n123 * sizeof(float));
+	v = (float*)malloc(n123 * sizeof(float));
+	p = (float*)malloc(n123 * sizeof(float));
+	
+
+    if (*sp != n123)
+    {
+    	printf("Dimension mismatch, N_input = %d, N_model = %d \n", *sp, n123);
+    	return NULL;
+    }
+    
+    /*reading velocity*/
+    for (i=0; i<n123; i++)
+    {
+        v[i]=*((float*)PyArray_GETPTR1(arr1,i));
+        v[i] = 1./(v[i]*v[i]);
+    }
+    
+    fastmarch_init (n3,n2,n1);
+ 
+    /* loop over shots */
+    nshot=1;
+    for( is = 0; is < nshot; is++) {
+	fastmarch(t,v,p, plane,
+		      n3,n2,n1,
+		      o3,o2,o1,
+		      d3,d2,d1,
+		      s[is][2],s[is][1],s[is][0], 
+		      b3,b2,b1,
+		      order);
+    }
+
+    int i1,i2,i3,if2d,iflip;
+    float grad1,grad2,grad3;
+    float cPI,cRPD;
+    
+    cPI = 4. * atan(1.); /* PI */
+    cRPD = cPI / 180.; /* radians per degree */
+    
+    /*initialization of takeoff angle*/
+    for(i=0;i<n123*nshot;i++)
+    {dip[i]=200;azim[i]=400;}
+    
+    if(n1==1 || n2==1)
+    {
+    if2d=1;	
+    }else{
+    if2d=0; 
+    }
+    
+    if(if2d==0) /*3D version*/
+    for(is=0;is<nshot;is++)
+	for(i1=1;i1<n1-1;i1++) /*x*/
+	for(i2=1;i2<n2-1;i2++) /*y*/
+	for(i3=1;i3<n3-1;i3++) /*z*/
+    	{
+    	/*reverse sign to get take-off angle*/
+    	grad1=-((t[i1+i2*n1+i3*n1*n2+is*n1*n2*n3]-t[i1-1+i2*n1+i3*n1*n2+is*n1*n2*n3])/d1 + 
+    		  (t[i1+1+i2*n1+i3*n1*n2+is*n1*n2*n3]-t[i1+i2*n1+i3*n1*n2+is*n1*n2*n3])/d1)/2.0;
+    	
+    	grad2=-((t[i1+i2*n1+i3*n1*n2+is*n1*n2*n3]-t[i1+(i2-1)*n1+i3*n1*n2+is*n1*n2*n3])/d2 + 
+    		  (t[i1+(i2+1)*n1+i3*n1*n2+is*n1*n2*n3]-t[i1+i2*n1+i3*n1*n2+is*n1*n2*n3])/d2)/2.0;
+    	
+    	grad3=-((t[i1+i2*n1+i3*n1*n2+is*n1*n2*n3]-t[i1+i2*n1+(i3-1)*n1*n2+is*n1*n2*n3])/d3 + 
+    		  (t[i1+i2*n1+(i3+1)*n1*n2+is*n1*n2*n3]-t[i1+i2*n1+i3*n1*n2+is*n1*n2*n3])/d3)/2.0;
+    	
+    	/* calculate dip angle (range of 0 (down) to 180 (up)) */
+    	dip[i1+i2*n1+i3*n1*n2+is*n1*n2*n3]=atan2(sqrt(grad1 * grad1 + grad2 * grad2), -grad3) / cRPD;
+    	/* calculate azimuth angle (0 to 360) */
+    	azim[i1+i2*n1+i3*n1*n2+is*n1*n2*n3] = atan2(grad1, grad2) / cRPD;
+    	if (azim[i1+i2*n1+i3*n1*n2+is*n1*n2*n3] < 0.0)
+        	azim[i1+i2*n1+i3*n1*n2+is*n1*n2*n3] += 360.0;
+    	}
+    else /*2D version*/
+    for(is=0;is<nshot;is++)
+	for(i1=0;i1<n1;i1++) /*x*/
+	for(i2=0;i2<n2;i2++) /*y*/
+	for(i3=1;i3<n3-1;i3++) /*z*/
+    	{
+    	
+        if(n1==1)/*y direction*/
+        {
+        if(i2>=1 && i2<n2-1)
+    	grad2=-((t[i1+i2*n1+i3*n1*n2+is*n1*n2*n3]-t[i1+(i2-1)*n1+i3*n1*n2+is*n1*n2*n3])/d2 + 
+    		  (t[i1+(i2+1)*n1+i3*n1*n2+is*n1*n2*n3]-t[i1+i2*n1+i3*n1*n2+is*n1*n2*n3])/d2)/2.0;/*y*/
+        }
+        
+        if(n2==1)/*x direction*/
+        {
+        if(i1>=1 && i1<n1-1)
+    	grad1=-((t[i1+i2*n1+i3*n1*n2+is*n1*n2*n3]-t[i1-1+i2*n1+i3*n1*n2+is*n1*n2*n3])/d1 + 
+    		  (t[i1+1+i2*n1+i3*n1*n2+is*n1*n2*n3]-t[i1+i2*n1+i3*n1*n2+is*n1*n2*n3])/d1)/2.0;/*x*/
+        }
+    	
+    	grad3=-((t[i1+i2*n1+i3*n1*n2+is*n1*n2*n3]-t[i1+i2*n1+(i3-1)*n1*n2+is*n1*n2*n3])/d3 + 
+    		  (t[i1+i2*n1+(i3+1)*n1*n2+is*n1*n2*n3]-t[i1+i2*n1+i3*n1*n2+is*n1*n2*n3])/d3)/2.0;/*z*/
+    	
+    	/* calculate dip angle (range of 0 (down) to 180 (up)) */
+//     	dip[i1+i2*n1+i3*n1*n2+is*n1*n2*n3]=atan2(sqrt(grad1 * grad1 + grad2 * grad2), -grad3) / cRPD;
+//     	/* calculate azimuth angle (0 to 360) */
+//     	azim[i1+i2*n1+i3*n1*n2+is*n1*n2*n3] = atan2(grad1, grad2) / cRPD;
+//     	if (azim[i1+i2*n1+i3*n1*n2+is*n1*n2*n3] < 0.0)
+//         	azim[i1+i2*n1+i3*n1*n2+is*n1*n2*n3] += 360.0;
+//         	
+        
+        /* calculate dip angle (range of 0 (down) to 180 (up)) */
+//         dip = atan2(grady, -gradz) / cRPD;
+        
+        if(n1==1)/*y direction*/
+        {
+        if(i2>=1 && i2<n2-1)
+        dip[i1+i2*n1+i3*n1*n2+is*n1*n2*n3] = atan2(grad2, grad3) / cRPD;
+        }
+        if(n2==1)/*x direction*/
+        {
+        if(i1>=1 && i1<n1-1)
+        dip[i1+i2*n1+i3*n1*n2+is*n1*n2*n3] = atan2(grad1, grad3) / cRPD;
+        }
+        
+        iflip = 0;
+        if (dip[i1+i2*n1+i3*n1*n2+is*n1*n2*n3] > 180.0) {
+            dip[i1+i2*n1+i3*n1*n2+is*n1*n2*n3] = dip[i1+i2*n1+i3*n1*n2+is*n1*n2*n3] - 180.0;
+            iflip = 1;
+        } else if (dip[i1+i2*n1+i3*n1*n2+is*n1*n2*n3] < 0.0) {
+            dip[i1+i2*n1+i3*n1*n2+is*n1*n2*n3] = -dip[i1+i2*n1+i3*n1*n2+is*n1*n2*n3];
+            iflip = 1;
+        }
+        /* calculate azimuth polarity (1 or -1) relative to pos Y dir */
+        azim[i1+i2*n1+i3*n1*n2+is*n1*n2*n3] = iflip ? -1.0 : 1.0;
+        	
+    	}
+    
+    
+    /*Below is the output part*/
+    PyArrayObject *vecout;
+	npy_intp dims[2];
+	dims[0]=n1*n2*n3*nshot*3;dims[1]=1;
+	/* Parse tuples separately since args will differ between C fcns */
+	/* Make a new double vector of same dimension */
+	vecout=(PyArrayObject *) PyArray_SimpleNew(1,dims,NPY_FLOAT);
+	
+	for(i=0;i<n1*n2*n3*nshot;i++)
+		(*((float*)PyArray_GETPTR1(vecout,i))) = t[i];
+	for(i=0;i<n1*n2*n3*nshot;i++)
+		(*((float*)PyArray_GETPTR1(vecout,i+n1*n2*n3*nshot))) = dip[i];
+	for(i=0;i<n1*n2*n3*nshot;i++)
+		(*((float*)PyArray_GETPTR1(vecout,i+n1*n2*n3*nshot*2))) = azim[i];
+		
+	return PyArray_Return(vecout);
+	
+}
+
+
 static PyObject *eikonalc_multishots(PyObject *self, PyObject *args){
 
     /*Below is the input part*/
@@ -898,6 +1127,188 @@ static PyObject *eikonalc_multishots(PyObject *self, PyObject *args){
 	
 }
 
+static PyObject *eikonalc_multishots_angle(PyObject *self, PyObject *args){
+	/*also calculate take off angle*/
+	/* following CalcAnglesGradient() in GridLib.c */
+	
+    /*Below is the input part*/
+    float f4,f5,f6,f7,f8,f9;
+    int f10,f11,f12,f13,f14;
+    
+	/**initialize data input**/
+    PyObject *arg1=NULL;
+    PyObject *arr1=NULL;
+    int nd, nd2;
+    
+    PyObject *f1=NULL;
+    PyObject *f2=NULL;
+    PyObject *f3=NULL;
+    PyObject *arrf1=NULL;
+    PyObject *arrf2=NULL;
+    PyObject *arrf3=NULL;
+
+	PyArg_ParseTuple(args, "OOOOffffffiiiii", &arg1, &f1, &f2, &f3, &f4, &f5, &f6, &f7, &f8, &f9, &f10, &f11, &f12, &f13, &f14);
+
+    int b1, b2, b3, n1, n2, n3, nshot, ndim, i, is,order,n123, *p, verb;
+    float br1, br2, br3, o1, o2, o3, d1, d2, d3;
+    float **s, *t, *v, *dip, *azim;
+    float *x, *y, *z;
+    bool plane[3];
+    
+	o1=f4;
+	o2=f5;
+	o3=f6;
+	
+	d1=f7;
+	d2=f8;
+	d3=f9;
+	
+	n1=f10;
+	n2=f11;
+	n3=f12;
+	
+	order=f13;
+	verb=f14; /*verbosity*/
+    
+    arr1 = PyArray_FROM_OTF(arg1, NPY_FLOAT, NPY_IN_ARRAY);
+    arrf1 = PyArray_FROM_OTF(f1, NPY_FLOAT, NPY_IN_ARRAY);
+    arrf2 = PyArray_FROM_OTF(f2, NPY_FLOAT, NPY_IN_ARRAY);
+    arrf3 = PyArray_FROM_OTF(f3, NPY_FLOAT, NPY_IN_ARRAY);
+
+    nd=PyArray_NDIM(arr1);
+    nd2=PyArray_NDIM(arrf1);
+    
+    npy_intp *sp=PyArray_SHAPE(arr1);
+    npy_intp *spxyz=PyArray_SHAPE(arrf1);
+    nshot=*spxyz;
+
+	br1=d1;
+	br2=d2;
+	br3=d3;
+	plane[2]=false;
+	plane[1]=false;
+	plane[0]=false;
+	b1= plane[2]? n1: (int) (br1/d1+0.5); 
+	b2= plane[1]? n2: (int) (br2/d2+0.5);
+	b3= plane[0]? n3: (int) (br3/d3+0.5); 
+
+
+    if( b1<1 ) b1=1;  
+    if( b2<1 ) b2=1;  
+    if( b3<1 ) b3=1;
+
+	ndim = 3; 
+    s = (float**)malloc(nshot * sizeof(float*));
+    for (int i = 0; i < nshot; i++)
+        s[i] = (float*)malloc(ndim * sizeof(float));
+	
+
+    n123 = n1*n2*n3;
+
+	t = (float*)malloc(n123*nshot * sizeof(float));
+	dip = (float*)malloc(n123*nshot * sizeof(float));
+	azim = (float*)malloc(n123*nshot * sizeof(float));
+	v = (float*)malloc(n123 * sizeof(float));
+	p = (float*)malloc(n123 * sizeof(float));
+
+	x = (float*)malloc(nshot * sizeof(float));
+	y = (float*)malloc(nshot * sizeof(float));
+	z = (float*)malloc(nshot * sizeof(float));
+
+
+    if (*sp != n123)
+    {
+    	printf("Dimension mismatch, N_input = %d, N_model = %d\n", *sp, n123);
+    	return NULL;
+    }
+    
+    /*reading velocity*/
+    for (i=0; i<n123; i++)
+    {
+        v[i]=*((float*)PyArray_GETPTR1(arr1,i));
+        v[i] = 1./(v[i]*v[i]);
+    }
+
+	/*reading xyz*/
+    for (i=0; i<nshot; i++)
+    {
+        s[i][0]=*((float*)PyArray_GETPTR1(arrf1,i));
+        s[i][1]=*((float*)PyArray_GETPTR1(arrf2,i));
+        s[i][2]=*((float*)PyArray_GETPTR1(arrf3,i));
+    }
+	
+    fastmarch_init (n3,n2,n1);
+ 
+    /* loop over shots */
+    for( is = 0; is < nshot; is++) {
+	if(verb) printf("shot %d of %d;\n",is+1,nshot);
+	fastmarch(t+is*n123,v,p, plane,
+		      n3,n2,n1,
+		      o3,o2,o1,
+		      d3,d2,d1,
+		      s[is][2],s[is][1],s[is][0], 
+		      b3,b2,b1,
+		      order);
+    }
+    
+    
+    int i1,i2,i3;
+    float grad1,grad2,grad3;
+    float cPI,cRPD;
+    
+    cPI = 4. * atan(1.); /* PI */
+    cRPD = cPI / 180.; /* radians per degree */
+    
+    /*initialization of takeoff angle*/
+    for(i=0;i<n123*nshot;i++)
+    {dip[i]=200;azim[i]=400;}
+    
+    
+    for(is=0;is<nshot;is++)
+	for(i1=1;i1<n1-1;i1++) /*x*/
+	for(i2=1;i2<n2-1;i2++) /*y*/
+	for(i3=1;i3<n3-1;i3++) /*z*/
+    	{
+    	/*reverse sign to get take-off angle*/
+    	grad1=-((t[i1+i2*n1+i3*n1*n2+is*n1*n2*n3]-t[i1-1+i2*n1+i3*n1*n2+is*n1*n2*n3])/d1 + 
+    		  (t[i1+1+i2*n1+i3*n1*n2+is*n1*n2*n3]-t[i1+i2*n1+i3*n1*n2+is*n1*n2*n3])/d1)/2.0;
+    	
+    	grad2=-((t[i1+i2*n1+i3*n1*n2+is*n1*n2*n3]-t[i1+(i2-1)*n1+i3*n1*n2+is*n1*n2*n3])/d2 + 
+    		  (t[i1+(i2+1)*n1+i3*n1*n2+is*n1*n2*n3]-t[i1+i2*n1+i3*n1*n2+is*n1*n2*n3])/d2)/2.0;
+    	
+    	grad3=-((t[i1+i2*n1+i3*n1*n2+is*n1*n2*n3]-t[i1+i2*n1+(i3-1)*n1*n2+is*n1*n2*n3])/d3 + 
+    		  (t[i1+i2*n1+(i3+1)*n1*n2+is*n1*n2*n3]-t[i1+i2*n1+i3*n1*n2+is*n1*n2*n3])/d3)/2.0;
+    	
+    	/* calculate dip angle (range of 0 (down) to 180 (up)) */
+    	dip[i1+i2*n1+i3*n1*n2+is*n1*n2*n3]=atan2(sqrt(grad1 * grad1 + grad2 * grad2), -grad3) / cRPD;
+    	/* calculate azimuth angle (0 to 360) */
+    	azim[i1+i2*n1+i3*n1*n2+is*n1*n2*n3] = atan2(grad1, grad2) / cRPD;
+    	if (azim[i1+i2*n1+i3*n1*n2+is*n1*n2*n3] < 0.0)
+        	azim[i1+i2*n1+i3*n1*n2+is*n1*n2*n3] += 360.0;
+    	}
+    
+    
+    
+    
+    /*Output are time, dip: 0 (down) to 180 (up), azimuth: 0 to 360*/
+    
+    /*Below is the output part*/
+    PyArrayObject *vecout;
+	npy_intp dims[2];
+	dims[0]=n1*n2*n3*nshot*3;dims[1]=1;
+	/* Parse tuples separately since args will differ between C fcns */
+	/* Make a new double vector of same dimension */
+	vecout=(PyArrayObject *) PyArray_SimpleNew(1,dims,NPY_FLOAT);
+	for(i=0;i<n1*n2*n3*nshot;i++)
+		(*((float*)PyArray_GETPTR1(vecout,i))) = t[i];
+	for(i=0;i<n1*n2*n3*nshot;i++)
+		(*((float*)PyArray_GETPTR1(vecout,i+n1*n2*n3*nshot))) = dip[i];
+	for(i=0;i<n1*n2*n3*nshot;i++)
+		(*((float*)PyArray_GETPTR1(vecout,i+n1*n2*n3*nshot*2))) = azim[i];
+	
+	return PyArray_Return(vecout);
+	
+}
 
 static PyObject *eikonalc_surf(PyObject *self, PyObject *args){
 
@@ -1304,7 +1715,9 @@ static char eikonalc_document[] = "Document stuff for eikonal...";
 // function_name, function, METH_VARARGS flag, function documents
 static PyMethodDef functions[] = {
   {"eikonalc_oneshot", eikonalc_oneshot, METH_VARARGS, eikonalc_document},
+  {"eikonalc_oneshot_angle", eikonalc_oneshot_angle, METH_VARARGS, eikonalc_document},
   {"eikonalc_multishots", eikonalc_multishots, METH_VARARGS, eikonalc_document},
+  {"eikonalc_multishots_angle",eikonalc_multishots_angle, METH_VARARGS, eikonalc_document},
   {"eikonalc_surf", eikonalc_surf, METH_VARARGS, eikonalc_document},
   {"eikonalc_oneshot_rtp", eikonalc_oneshot_rtp, METH_VARARGS, eikonalc_document},
   {"eikonalc_multishots_rtp", eikonalc_multishots_rtp, METH_VARARGS, eikonalc_document},
